//
// MavRaise (c) Edwin Groothuis, edwin@mavetju.org
//
// Parts of this code (well, everything except for main()) is stolen
// from dsimple.c of the XFree86 project (xc/programs/xlsfonts)
//
/*

Copyright 1993, 1998  The Open Group

All Rights Reserved.

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of The Open Group shall
not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization
from The Open Group.
*/


#include <X11/Xlib.h>
#include <stdio.h>

Display *	dpy;
Window		w;
char *		filename;
int		screen;

Display *Open_Display(char *display_name) {
    if ((dpy=XOpenDisplay(display_name))==NULL) {
	fprintf(stderr,"%s: unable to open display '%s'\n",
		filename,XDisplayName(display_name));
	exit(0);
    }

    return(dpy);
}


char *Get_Display_Name(int *pargc,char **argv) {
    int argc = *pargc;
    char **pargv = argv+1;
    char *displayname = NULL;
    int i;

    for (i = 1; i < argc; i++) {
	char *arg = argv[i];

	if (!strcmp (arg, "-display") || !strcmp (arg, "-d")) {
	    displayname = argv[i];
	    *pargc -= 2;
	    continue;
	}
	if (!strcmp(arg,"-")) {
	    while (i<argc)
		*pargv++ = argv[i++];
	    break;
	}
	*pargv++ = arg;
    }

    *pargv = NULL;
    return (displayname);
}


void Setup_Display_And_Screen(int *argc,char **argv) {
    dpy=Open_Display(Get_Display_Name(argc,argv));
}


Window Window_With_Name(Display *dpy,Window top,char *name) {
    Window *children, dummy;
    unsigned int nchildren;
    int i;
    Window w=0;
    char *window_name;

    if (XFetchName(dpy, top, &window_name) && !strcmp(window_name, name))
        return(top);
                        
    if (!XQueryTree(dpy, top, &dummy, &dummy, &children, &nchildren))
        return(0);
         
    for (i=0; i<nchildren; i++) {
	w = Window_With_Name(dpy, children[i], name);
	if (w)
	    break;
    }
    if (children) XFree ((char *)children);
    return(w);
}


Window Select_Window_Args(int *rargc,char **argv) {
#define ARGC (*rargc)
    int nargc=1;
    int argc;
    char **nargv;
    Window w=0;

    nargv = argv+1; argc = ARGC;
#define OPTION argv[0]
#define NXTOPTP ++argv, --argc>0
#define COPYOPT nargv++[0]=OPTION, nargc++

    while (NXTOPTP) {
	if (!strcmp(OPTION, "-")) {
	    COPYOPT;
	    while (NXTOPTP)
	    	COPYOPT;
	    break;
	}
	if (!strcmp(OPTION, "-root")) {
	    w=RootWindow(dpy, screen);
	    continue;
	}
	if (!strcmp(OPTION, "-name")) {
	    ++argv;--argc;
	    w=Window_With_Name(dpy, RootWindow(dpy, screen), OPTION);
	    if (!w) {
		fprintf(stderr,"No window with name %s exists!",OPTION);
		exit(1);
	    }
	    continue;
	}
	if (!strcmp(OPTION, "-id")) {
	    ++argv;--argc;
	    w=0;
	    sscanf(OPTION, "0x%lx", &w);
	    if (!w)
		sscanf(OPTION, "%ld", &w);
	    if (!w)
		fprintf(stderr,"Invalid window id format: %s.", OPTION);
	    continue;
	}
	COPYOPT;
    }
    ARGC = nargc;

    return(w);
}

int main(int argc,char **argv) {

    filename=argv[0];

    Setup_Display_And_Screen(&argc, argv);
    if ((w=Select_Window_Args(&argc, argv))==NULL) {
	perror("Cannot find window");
	return 1;
    }

    XRaiseWindow(dpy,w);
    XCloseDisplay(dpy);

    return 0;
}
